
import unittest
from pyspark.sql import SparkSession
import freeforestml as ff

class TestparkCut(unittest.TestCase):
    """Check that cut object can operate on PySpark dataframes"""
    def setUp(self):
        """Create or get spark session"""
        self.spark = SparkSession.builder.master("local").getOrCreate()
        self.sc = self.spark.sparkContext

    def test_func(self):
        """Check that lambda-cut can be applied to a spark dataframe"""
        rdd = self.sc.parallelize([(x, x**2) for x in range(100)])
        df = rdd.toDF(["x", "y"])

        cut = ff.Cut(lambda d: d.y < 100)
        selected = cut(df).count()
        self.assertEqual(selected, 10)

    def test_and(self):
        """Check that derived cuts can be used on a pyspark dataframe"""
        rdd = self.sc.parallelize([(x, x**2) for x in range(100)])
        df = rdd.toDF(["x", "y"])

        cut_y = ff.Cut(lambda d: d.y < 1000)
        cut_x = ff.Cut(lambda d: d.x > 2)
        cut = cut_x & cut_y

        selected = cut(df).count()
        self.assertEqual(selected, 29)

    def test_or(self):
        """Check that derived cuts can be used on a pyspark dataframe"""
        rdd = self.sc.parallelize([(x, x**2) for x in range(100)])
        df = rdd.toDF(["x", "y"])

        cut_y = ff.Cut(lambda d: d.y < 1000)
        cut_x = ff.Cut(lambda d: d.x > 2)
        cut = cut_x | cut_y

        selected = cut(df).count()
        self.assertEqual(selected, 29)

    def test_or(self):
        """Check that derived cuts can be used on a pyspark dataframe"""
        rdd = self.sc.parallelize([(x, x**2) for x in range(100)])
        df = rdd.toDF(["x", "y"])

        cut_y = ff.Cut(lambda d: d.y < 1000)
        cut_x = ff.Cut(lambda d: d.x > 2)
        cut = cut_x | cut_y

        selected = cut(df).count()
        self.assertEqual(selected, 100)

    def test_assign(self):
        """Check that a cut can assign a columns"""
        rdd = self.sc.parallelize([(x, x**2) for x in range(100)])
        df = rdd.toDF(["x", "y"])

        cut_new = ff.Cut(lambda d: d.x < 5, weight=lambda d: d.x * 2)
        sum_new = cut_new(df).agg({'weight': 'sum'}).collect()
        self.assertEqual(sum_new[0][0], 2 + 4 + 6 + 8)

    def test_overwrite(self):
        """Check that a cut can overwrite a columns"""
        rdd = self.sc.parallelize([(x, x**2) for x in range(100)])
        df = rdd.toDF(["x", "y"])

        cut_overwrite = ff.Cut(lambda d: d.x < 5, x=lambda d: d.x * 2)
        sum_overwrite = cut_overwrite(df).agg({'x': 'sum'}).collect()
        self.assertEqual(sum_overwrite[0][0], 2 + 4 + 6 + 8)
